//  Copyright (C) 2003 Koji Tsukada and Satoru Takabayashi
//      All rights reserved.
//      This is free software with ABSOLUTELY NO WARRANTY.
//
//  You can redistribute it and/or modify it under the terms of
//  the GNU General Public License version 2.

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

using System.Drawing.Imaging;
using System.Drawing.Drawing2D;

namespace FaceIcon

{
	public class IconForm : System.Windows.Forms.Form
	{
		private const int ICON_WIDTH = 48, ICON_HEIGHT = 48;

		enum ToolMode {NONE, SELECT, MOVE, PEN};
		enum MouseMode {DOWN_LEFT, DOWN_RIGHT, UP};

		#region class SelectRegion
		class SelectRegion
		{	
			private GraphicsPath path;
			private Point start, end;
			private Pen pen, penBold;
			private bool bold;
			private IconForm iconForm;
			private Point moveDefault;
			private ToolMode mode;

			public SelectRegion(IconForm form)
			{
				this.iconForm = form;
				this.pen = new Pen(Color.Black, 1.0F);
				this.pen.DashPattern =  new float[2] {2.0F, 2.0F};
				this.penBold = new Pen(Color.Black, 2.0F);
				this.penBold.DashPattern =  new float[2] {2.0F, 2.0F};
				this.bold = false;
			}

			public bool Bold
			{
				get{return this.bold;}
			}

			public GraphicsPath Path
			{
				get{return this.path;}
			}

			public Point Start
			{
				get{return this.start;}
			}

			public Point End
			{
				get{return this.end;}
			}
			
			public Pen Pen
			{
				get{return this.pen;}
			}

			public Pen PenBold
			{
				get{return this.penBold;}
			}

			public ToolMode Mode
			{
				set{this.mode = value; }
				get{return this.mode;  }
			}

			public void CreateStartPoint(Point p)
			{
				//IpXNA
				Point resizeBmpPoint = iconForm.resizeBmpPoint;
				Bitmap resizeBmp = iconForm.resizeBmp;
				this.path = null;
				
				iconForm.UpdateGraphics();

				//}EX̉ꂽʒuIJn_ƂĐݒ
				this.start = new Point(p.X - resizeBmpPoint.X, p.Y - resizeBmpPoint.Y);
				if(this.start.X < 0)
					this.start.X = 0;
				else if (this.start.X > resizeBmp.Width)
					this.start.X = resizeBmp.Width;
				if(this.start.Y < 0)
					this.start.Y = 0;
				else if (this.start.Y > resizeBmp.Height)
					this.start.Y = resizeBmp.Height;				
			}

			public void UpdateSelection(Point p)
			{
				Point resizeBmpPoint = iconForm.resizeBmpPoint;
				Bitmap resizeBmp = iconForm.resizeBmp;

				//}EXhbOĂ΁CړʒuII_ƂčXV
				this.end = new Point(p.X - resizeBmpPoint.X, p.Y - resizeBmpPoint.Y);
				if(this.end.X < 0)
					this.end.X = 0;
				else if (this.end.X > resizeBmp.Width)
					this.end.X = resizeBmp.Width;
				if(this.end.Y < 0)
					this.end.Y = 0;
				else if (this.end.Y > resizeBmp.Height)
					this.end.Y = resizeBmp.Height;

				//Ï悪`ɂȂ悤ɁCII_XV
				if(Math.Abs(this.end.X - this.start.X) >= 
					Math.Abs(this.end.Y - this.start.Y))
				{
					if(this.end.X >= this.start.X)
					{
						this.end.X = this.start.X +
							Math.Abs(this.end.Y - this.start.Y);
					}
					else
					{
						this.end.X = this.start.X -
							Math.Abs(this.end.Y - this.start.Y);

					}
				}
				else
				{
					if(this.end.Y >= this.start.Y)
					{
						this.end.Y = this.start.Y + 
							Math.Abs(this.end.X - this.start.X);
					}
					else
					{
						this.end.Y = this.start.Y -
							Math.Abs(this.end.X - this.start.X);
					}
				}
						
				//Ï̃pX쐬
				this.path = new GraphicsPath();
				this.path.AddLine(this.start.X, this.start.Y, 
					this.start.X, this.end.Y);
				this.path.AddLine(this.start.X, this.end.Y, 
					this.end.X, this.end.Y);
				this.path.AddLine(this.end.X, this.end.Y, 
					this.end.X, this.start.Y);
				this.path.AddLine(this.end.X, this.start.Y, 
					this.start.X, this.start.Y);
				this.path.CloseFigure();
						
				this.bold = false;
				iconForm.UpdateGraphics();
			}

			public void CreateMovePoint(Point p)
			{
				Bitmap resizeBmp = iconForm.resizeBmp;
				Point resizeBmpPoint = iconForm.resizeBmpPoint;
				Point temp = new Point();

				temp.X = p.X;
				temp.Y = p.Y;
				
				Point center = new Point();
				center.X = (int)((start.X + end.X) / 2.0);
				center.Y = (int)((start.Y + end.Y) / 2.0);
				
				if(temp.X < 0)
					temp.X = 0;
				else if (temp.X > resizeBmp.Width )
					temp.X = resizeBmp.Width;
				if(temp.Y < 0)
					temp.Y = 0;
				else if (temp.Y > resizeBmp.Height )
					temp.Y = resizeBmp.Height;		

				//}EX̉ꂽʒuIJn_ƂĐݒ
				this.moveDefault = new Point();
				this.moveDefault.X = (center.X - temp.X + resizeBmpPoint.X);
				this.moveDefault.Y = (center.Y - temp.Y + resizeBmpPoint.Y);
			}


			//Ï̈ړ
			public void UpdateMoveSelection(Point p)
			{
				int x,y, width, height;
				Bitmap resizeBmp = iconForm.resizeBmp;
				Point resizeBmpPoint = iconForm.resizeBmpPoint;
				Point originalPoint, translatePoint;
				Matrix translateMatrix;
				
				originalPoint = new Point((int)((this.start.X + this.end.X)/2),
					(int) ((this.start.Y + this.end.Y) / 2));

				x = p.X - resizeBmpPoint.X + moveDefault.X;
				y = p.Y - resizeBmpPoint.Y + moveDefault.Y;
				width =  (int)(Math.Abs(this.start.X - this.end.X));
				height = (int)(Math.Abs(this.start.Y - this.end.Y));

				translatePoint = new Point(x, y);		

				if(translatePoint.X - (originalPoint.X - this.Start.X) < 0)
					translatePoint.X = (originalPoint.X - this.Start.X);
				else if (translatePoint.X + (originalPoint.X - this.Start.X) > resizeBmp.Width)
					translatePoint.X = resizeBmp.Width - (originalPoint.X - this.Start.X);
				if(translatePoint.Y - (originalPoint.Y - this.Start.Y)  < 0)
					translatePoint.Y = (originalPoint.Y - this.Start.Y);
				else if (translatePoint.Y + (originalPoint.Y - this.Start.Y) >  resizeBmp.Height)
					translatePoint.Y = resizeBmp.Height - (originalPoint.Y - this.Start.Y);
				

				translateMatrix = new Matrix();    
				translateMatrix.Translate(translatePoint.X - originalPoint.X, 
					translatePoint.Y - originalPoint.Y);
				
				this.path.Transform(translateMatrix);
				
				this.start.X = translatePoint.X - (int)(width / 2.0);
				this.start.Y = translatePoint.Y - (int)(height / 2.0);
				this.end.X = translatePoint.X + (int)(width / 2.0);
				this.end.Y = translatePoint.Y + (int)(height / 2.0);
				this.bold =true;
				iconForm.UpdateGraphics();	
			}

			//Ïm肷
			public void FixSelection()
			{
				if(this.path != null)
				{
					this.bold = true;
					iconForm.UpdateGraphics();
				}
			}

			public void SetPen(Color c)
			{
				this.pen = new Pen(c, 1.0F);
				this.pen.DashPattern =  new float[2] {2.0F, 2.0F};
				this.penBold = new Pen(c, 2.0F);
				this.penBold.DashPattern =  new float[2] {2.0F, 2.0F};
				iconForm.UpdateGraphics();
			}
		}
		
		#endregion

		#region class FillPoint
		class FillPoint
		{
			public Point point;
			public bool filled;
			public bool changed;
		}
		#endregion

		#region class EditMode
		class EditMode
		{				
			public ToolMode tool;
			public MouseMode mouse;
			public EditMode()
			{
				this.tool = ToolMode.NONE;
				this.mouse = MouseMode.UP;
			}
		}
		#endregion
		
		#region class GridMode
		class GridMode
		{
			public bool visible;
			public Pen pen;

			public GridMode()
			{
				this.visible = true;
				this.pen =  new Pen(Color.White, 0.5F);
			}
		}
		#endregion

		#region variables
		//I[h̑I͈
		SelectRegion selectRegion;
		EditMode mode;
		GridMode grid;

		//ꂽACRIuWFNg
		private Icon faceIcon = null;

		//sNZ
		public bool [,] filledPixels;

		//Undo/RedóC`Xg[NǗpHashtable.
		Hashtable dotHolder = new Hashtable();
		Hashtable strokeHolder = null;

		//Undo/RedoǗp̃JEg
		int undoCount, strokeCount;	
		
		//݂̃}EXʒu
		private Point currentMousePoint;				
		
		//IWĩrbg}bvƕ\prbg}bv
		Bitmap originalBmp = null, resizeBmp = null;
		
		//ؔ̃rbg}bvƕ\prbg}bv
		Bitmap clipBmp = null, resizeClipBmp = null;
		
		//rbg}bv̉摜TCŸk
		float resizeRate, resizeClipRate;
		
		//rbg}bv̐؂蔲̎n_
		Point resizeBmpPoint;

		//pictureBox1graphicsIuWFNg
		Graphics graphics;

		//y̑
		int penSize = 1;

		//ÕÿʒuiXg[N̕ۊǗpj
		Point pastPenPoint;

		//ACRt@C̓ǂݍ
		bool imageIsIcon = false;
		#endregion

		#region Definition of Windows.Forms
		private System.Windows.Forms.Button buttonToolOpen;
		private System.Windows.Forms.RadioButton radioButtonSelection;
		private System.Windows.Forms.PictureBox pictureBox1;
		private System.Windows.Forms.RadioButton radioButtonPen;
		private System.Windows.Forms.Button buttonCut;
		private System.Windows.Forms.Button buttonOK;
		private System.Windows.Forms.ToolTip toolTip1;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.Label label9;
		private System.Windows.Forms.Label label10;
		private System.Windows.Forms.Label label14;
		private System.Windows.Forms.Label label15;
		private System.Windows.Forms.Label label26;
		private System.Windows.Forms.Label label27;
		private System.Windows.Forms.Panel panelOpen;
		private System.Windows.Forms.Panel panelSelect;
		private System.Windows.Forms.Panel panelPen;
		private System.Windows.Forms.Panel panelCencel;
		private System.Windows.Forms.Label label29;
		private System.Windows.Forms.Label label28;
		private System.Windows.Forms.Button buttonCancel;
		private System.Windows.Forms.Label label7;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.Label label30;
		private System.Windows.Forms.Label label31;
		private System.Windows.Forms.Label label32;
		private System.Windows.Forms.Button buttonOpen2;
		private System.Windows.Forms.GroupBox groupBox2;
		private System.Windows.Forms.GroupBox groupBox3;
		private System.Windows.Forms.Label label34;
		private System.Windows.Forms.Label label33;
		private System.Windows.Forms.Button buttonBack;
		private System.Windows.Forms.Button buttonToolUndo;
		private System.Windows.Forms.Label label25;
		private System.Windows.Forms.Label label24;
		private System.Windows.Forms.Label label23;
		private System.Windows.Forms.Button buttonToolRedo;
		private System.Windows.Forms.Label label20;
		private System.Windows.Forms.Label label17;
		private System.Windows.Forms.Label label21;
		private System.Windows.Forms.RadioButton radioGridNone;
		private System.Windows.Forms.Label label22;
		private System.Windows.Forms.RadioButton radioGridBlack;
		private System.Windows.Forms.Label label19;
		private System.Windows.Forms.Label label18;
		private System.Windows.Forms.Label label16;
		private System.Windows.Forms.RadioButton radioGridWhite;
		private System.Windows.Forms.GroupBox groupBox1;
		private System.Windows.Forms.RadioButton radioPen3;
		private System.Windows.Forms.RadioButton radioPen2;
		private System.Windows.Forms.RadioButton radioPen1;
		private System.Windows.Forms.RadioButton radioPen4;
		private System.Windows.Forms.RadioButton radioAreaBlack;
		private System.Windows.Forms.RadioButton radioAreaWhite;
		private System.Windows.Forms.Label label11;
		private System.Windows.Forms.Label label13;
		private System.Windows.Forms.Label label12;
		private System.ComponentModel.IContainer components;
		#endregion

		#region Constructor
		public IconForm(Image image)
		{
			InitIconForm();
			imageIsIcon = true;
			LoadImage(image);
			mode.tool = ToolMode.SELECT;
			ChangeFormState(mode.tool);
		}

		public IconForm()
		{
			InitIconForm();
		}

		#endregion


		public Icon FaceIcon
		{
			get{return faceIcon;}
		}

		#region Other Methods
		//
		private void InitIconForm()
		{
			InitializeComponent();		

			mode = new EditMode();
			selectRegion = new SelectRegion(this);
			grid = new GridMode();
			filledPixels = new bool [ICON_WIDTH, ICON_HEIGHT];
		
			graphics = Graphics.FromImage(pictureBox1.Image);
			graphics.InterpolationMode = InterpolationMode.HighQualityBicubic;	//⊮[h̐ݒ
			
			//rbg}bvt@C̏ݒ
			originalBmp = new Bitmap(pictureBox1.Image);
			resizeBmp = new Bitmap(originalBmp);
			resizeBmpPoint = new Point(0,0);
			resizeRate = 1.0F;

			pastPenPoint = new Point(-1,-1);

			//tH[̕\ԂύX
			ChangeFormState(mode.tool);
		}


		//etH[R|[lg̕\ԂύX
		private void ChangeFormState(ToolMode tm)
		{
			switch(tm)
			{
				case ToolMode.NONE:
					panelOpen.Visible = true;
					panelCencel.Visible = true;
					panelPen.Visible = false;
					panelSelect.Visible = false;

					break;

				case ToolMode.SELECT:
					panelOpen.Visible = false;
					panelSelect.Visible = true;
					panelPen.Visible = false;
					break;
				case ToolMode.PEN:
					panelOpen.Visible = false;
					panelSelect.Visible = false;
					panelPen.Visible = true;

					break;
			}
		}


		//_CAO摜t@CI
		private void OpenImageDialog()
		{

			OpenFileDialog dialog = new OpenFileDialog();

			dialog.InitialDirectory = 
				Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
			dialog.Filter = "image files (*.bmp;*.jpg;*.gif;*.png)|*.bmp;*.jpg;*.gif;*.png |" + 
				"icon files (*.ico)|*.ico" ;
			dialog.RestoreDirectory = true;

			if (dialog.ShowDialog() == DialogResult.OK)
			{
				LoadImage(dialog.FileName);
				mode.tool = ToolMode.SELECT;
				ChangeFormState(mode.tool);
			}

		}


		//摜t@C̓ǂݍ݂ƃTCY
		private void LoadImage (string fileName)
		{
			try 
			{
				originalBmp = new Bitmap(Bitmap.FromFile(fileName));

				//hbOhbvꂽ摜ACRt@Cǂ
				if(String.Compare(fileName, fileName.Length - 4, ".ico", 0, 4,true) == 0)
				{
					imageIsIcon = true;
				}
				else
				{
					imageIsIcon = false;
				}

				//rbg}bvt@Cǂݍ݁CPictureBoxɂ킹ăTCY
				if(originalBmp.Width > originalBmp.Height)
				{
					resizeRate = (float)pictureBox1.Width / (float)originalBmp.Width;
					resizeBmp = new Bitmap(originalBmp, (int)(originalBmp.Width * resizeRate),
						(int)(originalBmp.Height * resizeRate));
				}
				else
				{
					resizeRate = (float)pictureBox1.Height / (float)originalBmp.Height;
					resizeBmp = new Bitmap(originalBmp, (int)(originalBmp.Width * resizeRate),
						(int)(originalBmp.Height * resizeRate));
				}
				
				//TCYꂽrbg}bv̌_擾
				resizeBmpPoint = new Point((pictureBox1.Width - resizeBmp.Width)/2, 
					(pictureBox1.Height - resizeBmp.Height)/2); 
				
				pictureBox1.Image = new Bitmap(resizeBmp);
				graphics = Graphics.FromImage(pictureBox1.Image);
				graphics.InterpolationMode = InterpolationMode.HighQualityBicubic;//⊮[h̐ݒ
			}
			catch (Exception e)
			{
				MessageBox.Show("ACR̓ǂݍ݂Ɏs܂" + "\n" + e.Message);
			}
		}


		//摜t@C̓ǂݍ݂ƃTCYiLoadImage(filename)Ƃقړlj
		private void LoadImage(Image image)
		{
			originalBmp = new Bitmap(image);

			//rbg}bvt@Cǂݍ݁CPictureBoxɂ킹ăTCY
			if(originalBmp.Width > originalBmp.Height)
			{
				resizeRate = (float)pictureBox1.Width / (float)originalBmp.Width;
				resizeBmp = new Bitmap(originalBmp, (int)(originalBmp.Width * resizeRate),
					(int)(originalBmp.Height * resizeRate));
			}
			else
			{
				resizeRate = (float)pictureBox1.Height / (float)originalBmp.Height;
				resizeBmp = new Bitmap(originalBmp, (int)(originalBmp.Width * resizeRate),
					(int)(originalBmp.Height * resizeRate));
			}
				
			//TCYꂽrbg}bv̌_擾
			resizeBmpPoint = new Point((pictureBox1.Width - resizeBmp.Width)/2, 
				(pictureBox1.Height - resizeBmp.Height)/2); 
				
			pictureBox1.Image = new Bitmap(resizeBmp);
			graphics = Graphics.FromImage(pictureBox1.Image);
			graphics.InterpolationMode = InterpolationMode.HighQualityBicubic;//⊮[h̐ݒ
		}


		//Ï̐؂ƕ\̈ւ̃TCYύX
		private void ClipSelection()
		{
			
			if(selectRegion.Path == null)
				return;
			
			Bitmap bmp;
			Point point;
			point = new Point(0,0);

			if(selectRegion.Start.X <= selectRegion.End.X)
				point.X = selectRegion.Start.X;
			else
				point.X = selectRegion.End.X;
			
			if(selectRegion.Start.Y <= selectRegion.End.Y)
				point.Y = selectRegion.Start.Y;
			else
				point.Y = selectRegion.End.Y;
			
			//Ï̉摜Vrbg}bv쐬
			bmp = originalBmp.Clone(new Rectangle(
				(int)(point.X / resizeRate), 
				(int)(point.Y / resizeRate), 
				(int)(Math.Abs(selectRegion.End.X - selectRegion.Start.X) / resizeRate),							(int)(Math.Abs(selectRegion.End.Y - selectRegion.Start.Y) / resizeRate)
				),
				PixelFormat.DontCare);
			
			clipBmp = new Bitmap(bmp, ICON_WIDTH, ICON_HEIGHT);
			resizeClipBmp = new Bitmap(clipBmp, pictureBox1.Width, pictureBox1.Height);
			resizeClipRate = (float)pictureBox1.Width / (float)ICON_WIDTH;
			
			pictureBox1.Image = new Bitmap(resizeClipBmp);
			graphics =  Graphics.FromImage(pictureBox1.Image);
			graphics.InterpolationMode = InterpolationMode.HighQualityBicubic;//⊮[h̐ݒ
			
			//ACRt@C̓hԂ
			if(imageIsIcon)
				initFillPoints();

			//Obh`
			grid.visible = true;
			DrawAllGrids(new Point(0,0));
			
			mode.tool = ToolMode.PEN;
			ChangeFormState(mode.tool);
		}


		//ACRt@C̓hԂD
		private void initFillPoints()
		{
	
			for(int x = 0; x< ICON_WIDTH;x++)
			{
				for(int y =0;y<ICON_HEIGHT;y++)
				{
					Color temp = clipBmp.GetPixel(x,y);

					if(clipBmp.GetPixel(x,y) == Color.FromArgb(0,0,0,0))
					{
						filledPixels[x,y] = true;
					}
				}
			}
		}


		//yXg[N̍쐬
		private void CreateStroke(Point p)
		{
			Hashtable fillRegion = new Hashtable();
			int x, y;
			int x_cnt=0, y_cnt=0, x_cnt_max =0, y_cnt_max =0, cnt_max =0;
			bool x_code =true , y_code = true;

			if(mode.mouse == MouseMode.UP)
			{
				UpdateGraphics();
				return;
			}
 
			if(strokeHolder == null)
			{
				strokeCount = 0;
				strokeHolder = new Hashtable();
			}

			x = p.X / ((int)resizeClipRate);
			y = p.Y / ((int)resizeClipRate);

			//⊮Kv肵C΂ꂽObh擾
			//}EXJ[\𑁂ꍇC肱ڂ邽߁C|CgԂ⊮
			if(pastPenPoint.X == -1)
			{
				pastPenPoint = new Point(x,y);
			}
			else
			{
				if(Math.Abs(pastPenPoint.X - x) > penSize || Math.Abs(pastPenPoint.Y -y) > penSize)
				{
					x_cnt_max = x - pastPenPoint.X;
					y_cnt_max = y - pastPenPoint.Y;

					if(x_cnt_max < 0)
						x_code = false;
					if(y_cnt_max < 0)
						y_code = false;

					x_cnt_max = Math.Abs(x_cnt_max);
					y_cnt_max = Math.Abs(y_cnt_max);
					cnt_max   = Math.Max(x_cnt_max, y_cnt_max);
					
				}
				pastPenPoint.X = x;
				pastPenPoint.Y = y;
			}

			//sNZ̓hԂ
			for(int cnt=0;cnt < cnt_max + 1;cnt++)
			{
				//ԂKvȏꍇ,x, y擾
				if(cnt_max >= 1)
				{
					if(x_cnt_max >= y_cnt_max)
					{
						if(x_code)
							x_cnt++;
						else
							x_cnt--;
	
						if(y_code)
							y_cnt = (int)(Math.Abs(x_cnt) * ((float)y_cnt_max / (float)x_cnt_max));
						else
							y_cnt = -(int)(Math.Abs(x_cnt) * ((float)y_cnt_max / (float)x_cnt_max));
					}
					else
					{
						if(y_code)
							y_cnt++;
						else
							y_cnt--;
	
						if(x_code)
							x_cnt =  (int)(Math.Abs(y_cnt) * ((float)x_cnt_max / (float)y_cnt_max));
						else
							x_cnt =  -(int)(Math.Abs(y_cnt) * ((float)x_cnt_max / (float)y_cnt_max));
					}
	
					x = pastPenPoint.X - x_cnt;
					y = pastPenPoint.Y - y_cnt;

					if(x<0)
						x=0;
					else if(x>=ICON_WIDTH)
						x=ICON_WIDTH-1;

					if(y<0)
						y=0;
					else if(y>=ICON_HEIGHT)
						y=ICON_HEIGHT-1;

				}
	
				//GridTCY{ԕ̃sNZhׂ^
				for(int i = 0;i < penSize;i++)
				{
					for(int j = 0;j < penSize;j++)
					{
						if (!(x+i < 0 || x+i >= ICON_WIDTH || y+j<0 || y+j>=ICON_HEIGHT))
						{
							FillPoint point = new FillPoint();
							point.point = new Point(x+i,y+j);
							if(!(fillRegion.Contains(cnt*(penSize*penSize)+i*penSize+j)))
							{
								if(mode.mouse == MouseMode.DOWN_LEFT)
								{
									point.filled = true;
									if(filledPixels[x+i, y+j])
										point.changed = false;
									else
										point.changed = true;

									fillRegion.Add(cnt*(penSize*penSize)+i*penSize+j,point);		
									filledPixels[x+i,y+j] = true;
								}
								else if(mode.mouse == MouseMode.DOWN_RIGHT)
								{
									point.filled = false;
									if(filledPixels[x+i, y+j])
										point.changed = true;
									else
										point.changed = false;

									fillRegion.Add(cnt*(penSize*penSize)+i*penSize+j,point);
									filledPixels[x+i,y+j] = false;
								}
							}
						}
					}
				}
			}

			//ύX_ȂStrokeHolderɓo^Ȃ
			bool changed = false;
			foreach (FillPoint fp in fillRegion.Values)
			{
				if(fp.changed == true)
				{
					changed = true;
					break;
				}
			}
			if(!(changed))
			{
				UpdateGraphics();
				return;
			}
			strokeHolder.Add(strokeCount, fillRegion);
			strokeCount++;
			UpdateGraphics();
			return;
		}

		//yXg[NUndoobt@ɒǉ
		private void CreateUndoBuffer()
		{
			//ЂƂύX_ȂUndo Bufferɓo^Ȃ
			if(strokeHolder == null || strokeHolder.Count < 1)
			{
				strokeHolder = null;
				return;
			}
		
			//d΁CߋBuffer폜
			if(dotHolder.Count >= undoCount)
			{
				for(int i=0;i<dotHolder.Count;i++)
				{
					if(i>=undoCount)
					{
						dotHolder.Remove(i);
					}
				}
			}
			
			//undoobt@ɒǉ
			dotHolder.Add(undoCount, strokeHolder);
			undoCount++;

			strokeHolder = null;

			return;						
		}


		//Undo̎s
		private void UndoStroke()
		{
			if(mode.tool != ToolMode.PEN || undoCount <= 0)
				return;

			Hashtable stroke = (Hashtable)dotHolder[undoCount - 1];

			foreach (Hashtable region in stroke.Values)
			{
				foreach (FillPoint fp in region.Values)
				{
					if(!(fp.changed))
						continue;

					if(fp.filled)
						filledPixels[fp.point.X,fp.point.Y] = false;
					else
						filledPixels[fp.point.X,fp.point.Y] = true;
				}
			}

			undoCount --;
			UpdateGraphics();
		}

		//Redo̎s
		private void RedoStroke()
		{
			if(mode.tool != ToolMode.PEN || undoCount >= dotHolder.Count)
				return;

			Hashtable stroke = (Hashtable)dotHolder[undoCount];

			foreach (Hashtable region in stroke.Values)
			{
				foreach (FillPoint fp in region.Values)
				{
					if(!(fp.changed))
						continue;

					if(fp.filled)
						filledPixels[fp.point.X,fp.point.Y] = true;
					else
						filledPixels[fp.point.X,fp.point.Y] = false;
				}
			}
			undoCount ++;

			UpdateGraphics();
		}


		//ĕ`
		private void UpdateGraphics()
		{
			switch(mode.tool)
			{
				case ToolMode.SELECT:
				case ToolMode.MOVE:
					
					graphics.Clear(pictureBox1.BackColor);
					graphics.DrawImage(resizeBmp,0,0);
					if(selectRegion.Path != null)
					{
						if(selectRegion.Bold)
							graphics.DrawPath(selectRegion.PenBold, selectRegion.Path);
						else
							graphics.DrawPath(selectRegion.Pen, selectRegion.Path);
					}
					pictureBox1.Refresh();
					
					break;
				case ToolMode.PEN:
					graphics.Clear(pictureBox1.BackColor);
					graphics.DrawImage(resizeClipBmp,0,0);
					//Obh̕`
					DrawAllGrids(currentMousePoint);
					break;
			}
			return;
		}


		//L[{[hɂV[gJbg
		private void IconForm_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			switch(mode.tool)
			{
				case ToolMode.SELECT:
				case ToolMode.MOVE:
				switch(e.KeyCode)
				{
					case Keys.O:
						OpenImageDialog();
						break;

					case Keys.B:
						selectRegion.SetPen(Color.Black);
						break;
						
					case Keys.W:
						selectRegion.SetPen(Color.White);
						break;

					case Keys.S:
						mode.tool = ToolMode.SELECT;
						pictureBox1.Cursor = Cursors.Cross;
						radioButtonSelection.Checked = true;
						break;

					case Keys.M:
						mode.tool = ToolMode.MOVE;
						pictureBox1.Cursor = Cursors.SizeAll;
						radioButtonSelection.Checked = true;
						break;
				}
					break;
				case ToolMode.PEN:
				switch(e.KeyCode)
				{
					case Keys.G:
						grid.visible = !(grid.visible);
						UpdateGraphics();
					
						break;
					case Keys.B:
						grid.pen = new Pen(Color.Black, 0.5F);
						UpdateGraphics();
						break;
						
					case Keys.W:
						grid.pen = new Pen(Color.White, 0.5F);
						UpdateGraphics();
						break;
					case Keys.U:
						UndoStroke();
						break;

					case Keys.R:
						RedoStroke();
						break;
					case Keys.D1:
						penSize = 1;
						UpdateGraphics();
						break;
					case Keys.D2:
						penSize = 2;
						UpdateGraphics();
						break;
					case Keys.D3:
						penSize = 3;
						UpdateGraphics();
						break;
					case Keys.D4:
						penSize = 4;
						UpdateGraphics();
						break;
				}
					break;
				default:
				switch(e.KeyCode)
				{
					case Keys.O:
						OpenImageDialog();
						break;
				}
					break;
			}
		}

		//ACR쐬Gȏ
		private void CreateFaceIcon()
		{
			Bitmap bmp = new Bitmap(clipBmp);

			//eF4bitɕύXC0,0,0ƂȂFuih~j
			for(int i=0;i<ICON_WIDTH;i++)
			{
				for(int j=0;j<ICON_HEIGHT;j++)
				{
					int r,g,b;

					r = (int)(clipBmp.GetPixel(i,j).R / 16);
					g = (int)(clipBmp.GetPixel(i,j).G / 16);
					b = (int)(clipBmp.GetPixel(i,j).B / 16);

					if(r == 0&& b==0 && g==0)
					{
						bmp.SetPixel(i,j,Color.FromArgb(16,16,16));
					}
				}
			}


			for(int i = 0; i < ICON_WIDTH; i++)
			{
				for(int j =0; j < ICON_HEIGHT; j++)
				{
					if(filledPixels[i,j] == true)
					{
						bmp.SetPixel(i,j,Color.Transparent);
					}
				}
			}

			faceIcon = Icon.FromHandle(bmp.GetHicon());

		}


		//valstartend͈̔͂ɂ邩`FbN
		private bool CheckInRange(int start, int end, int val)
		{
			if(start>=end)
			{
				if(start >= val && end <= val)
					return true;
				else
					return false;
			}
			else
			{
				if(start <= val && end >= val)
					return true;
				else
					return false;

			}
		}


		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		
		#endregion

		#region Methods for drawing Grids
		//Obh`̈Ȁ
		private void DrawAllGrids(Point p)
		{
			if(grid.visible)
				DrawGrids();
			FillSelectedGrids();
			DrawCurrentGrid(p);
			pictureBox1.Refresh();	
			return;
		}


		//݂̃Obhĕ`
		private void DrawCurrentGrid(Point point)
		{
			Pen p = new Pen(Color.Red, 0.5F);
			int x, y;

			x = (point.X / (int)resizeClipRate) * (int)resizeClipRate;
			y = (point.Y / (int)resizeClipRate) * (int)resizeClipRate;

			graphics.DrawRectangle(p, (int)x, (int)y, (int)resizeClipRate * penSize,
				(int)resizeClipRate * penSize);

			//pictureBox1.Refresh();	
		}


		//Obh`
		private void DrawGrids()
		{
			for(int i = 0; i < ICON_WIDTH; i++)
			{ 
				graphics.DrawLine(grid.pen, (int)(i * resizeClipRate), 0, 
					(int)(i * resizeClipRate),pictureBox1.Height);
			}
			for(int i =0; i < ICON_HEIGHT;i++)
			{
				graphics.DrawLine(grid.pen, 0,(int)(i * resizeClipRate), 
					pictureBox1.Width, (int)(i * resizeClipRate));
			}
		}


		//ḧׂ`(ƊDF̊iq)
		private void FillSelectedGrids()
		{
			Brush b;
			Brush white = Brushes.White;
			Brush gray = Brushes.LightGray;
			
			for(int i = 0; i < ICON_WIDTH; i++)
			{ 
				for(int j =0; j < ICON_HEIGHT;j++)
				{
					if(filledPixels[i,j] == true)
					{
						if(i % 2 ==0)
						{
							if(j%2 ==0)
								b = white;
							else
								b = gray;
						}
						else
						{
							if(j%2 ==0)
								b = gray;
							else
								b = white;

						}
						graphics.FillRectangle(b,(int)(i * resizeClipRate), (int)(j * resizeClipRate),
							(int)resizeClipRate, (int)resizeClipRate);
					}
				}
			}
			//pictureBox1.Refresh();	
		}

		#endregion

		#region Methods for Checking Mouse Events
		//}EX{^ꂽƂ̏
		private void pictureBox1_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			switch(mode.tool)
			{
				case (ToolMode.SELECT):
				case (ToolMode.MOVE):	
					if(e.Button == MouseButtons.Left)
					{
						if(CheckInRange(selectRegion.Start.X + resizeBmpPoint.X, 
							selectRegion.End.X + resizeBmpPoint.X, e.X) 
							&&
							CheckInRange(selectRegion.Start.Y  + resizeBmpPoint.Y, 
							selectRegion.End.Y + resizeBmpPoint.Y, e.Y))
						{
							mode.mouse = MouseMode.DOWN_LEFT;
							if(selectRegion.Path != null)
								Cursor.Current = Cursors.SizeAll;
							selectRegion.CreateMovePoint(new Point(e.X, e.Y));
							mode.tool = ToolMode.MOVE;						
						}
						else
						{
							selectRegion.CreateStartPoint(new Point(e.X,e.Y));
							mode.mouse = MouseMode.DOWN_LEFT;
							mode.tool = ToolMode.SELECT;
							Cursor.Current = Cursors.Arrow;
						}
					}
					break;

				case (ToolMode.PEN):
					if(e.Button == MouseButtons.Left)
						mode.mouse = MouseMode.DOWN_LEFT;
					else if(e.Button == MouseButtons.Right)
						mode.mouse = MouseMode.DOWN_RIGHT;
				
					CreateStroke(new Point(e.X, e.Y));

					break;
			}
				
		}

		//}EXړꂽƂ̏
		private void pictureBox1_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			//int x, y, width, height;

			switch(mode.tool){
				case (ToolMode.SELECT):
				case (ToolMode.MOVE):
					if (mode.tool == ToolMode.SELECT && mode.mouse == MouseMode.DOWN_LEFT)
					{
						selectRegion.UpdateSelection(new Point (e.X, e.Y));
						return;
					}
					//Ï悪C}EXhbOĂꍇ͑Ï̈ړ
					if(mode.tool == ToolMode.MOVE && selectRegion.Path != null 
							&& mode.mouse == MouseMode.DOWN_LEFT)
						selectRegion.UpdateMoveSelection(new Point(e.X, e.Y));

					if(CheckInRange(selectRegion.Start.X + resizeBmpPoint.X, 
						selectRegion.End.X + resizeBmpPoint.X, e.X) 
						&&
						CheckInRange(selectRegion.Start.Y  + resizeBmpPoint.Y, 
						selectRegion.End.Y + resizeBmpPoint.Y, e.Y)
						&& selectRegion.Path != null)
					{
						Cursor.Current = Cursors.SizeAll;
					}
					else
					{
						Cursor.Current = Cursors.Default;
					}	
					break;
				case (ToolMode.PEN):
					currentMousePoint = new Point(e.X, e.Y);
					CreateStroke(currentMousePoint) ;
					break;
			}
		
		}

		//}EX{^ꂽƂ̏
		private void pictureBox1_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			switch(mode.tool)
			{
				case (ToolMode.SELECT):	
				case (ToolMode.MOVE):
					mode.mouse = MouseMode.UP;
					selectRegion.FixSelection();
					break;

				case(ToolMode.PEN):
					CreateUndoBuffer();
					pastPenPoint = new Point(-1,-1);
					mode.mouse = MouseMode.UP;
					break;
			}		
		}
		
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// fUCi T|[gɕKvȃ\bhłB̃\bh̓e
		/// R[h GfB^ŕύXȂłB
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(IconForm));
			this.pictureBox1 = new System.Windows.Forms.PictureBox();
			this.buttonToolOpen = new System.Windows.Forms.Button();
			this.radioButtonSelection = new System.Windows.Forms.RadioButton();
			this.radioButtonPen = new System.Windows.Forms.RadioButton();
			this.buttonCut = new System.Windows.Forms.Button();
			this.buttonOK = new System.Windows.Forms.Button();
			this.toolTip1 = new System.Windows.Forms.ToolTip(this.components);
			this.buttonCancel = new System.Windows.Forms.Button();
			this.buttonOpen2 = new System.Windows.Forms.Button();
			this.buttonBack = new System.Windows.Forms.Button();
			this.buttonToolUndo = new System.Windows.Forms.Button();
			this.buttonToolRedo = new System.Windows.Forms.Button();
			this.radioGridNone = new System.Windows.Forms.RadioButton();
			this.radioGridBlack = new System.Windows.Forms.RadioButton();
			this.radioGridWhite = new System.Windows.Forms.RadioButton();
			this.radioPen3 = new System.Windows.Forms.RadioButton();
			this.radioPen2 = new System.Windows.Forms.RadioButton();
			this.radioPen1 = new System.Windows.Forms.RadioButton();
			this.radioPen4 = new System.Windows.Forms.RadioButton();
			this.radioAreaBlack = new System.Windows.Forms.RadioButton();
			this.radioAreaWhite = new System.Windows.Forms.RadioButton();
			this.panelOpen = new System.Windows.Forms.Panel();
			this.label4 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.label1 = new System.Windows.Forms.Label();
			this.panelSelect = new System.Windows.Forms.Panel();
			this.label11 = new System.Windows.Forms.Label();
			this.label13 = new System.Windows.Forms.Label();
			this.label12 = new System.Windows.Forms.Label();
			this.groupBox2 = new System.Windows.Forms.GroupBox();
			this.label7 = new System.Windows.Forms.Label();
			this.label32 = new System.Windows.Forms.Label();
			this.label8 = new System.Windows.Forms.Label();
			this.label30 = new System.Windows.Forms.Label();
			this.label31 = new System.Windows.Forms.Label();
			this.label10 = new System.Windows.Forms.Label();
			this.label9 = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.panelPen = new System.Windows.Forms.Panel();
			this.label25 = new System.Windows.Forms.Label();
			this.label24 = new System.Windows.Forms.Label();
			this.label23 = new System.Windows.Forms.Label();
			this.label20 = new System.Windows.Forms.Label();
			this.label17 = new System.Windows.Forms.Label();
			this.label21 = new System.Windows.Forms.Label();
			this.label22 = new System.Windows.Forms.Label();
			this.label19 = new System.Windows.Forms.Label();
			this.label18 = new System.Windows.Forms.Label();
			this.label16 = new System.Windows.Forms.Label();
			this.groupBox1 = new System.Windows.Forms.GroupBox();
			this.groupBox3 = new System.Windows.Forms.GroupBox();
			this.label34 = new System.Windows.Forms.Label();
			this.label33 = new System.Windows.Forms.Label();
			this.label26 = new System.Windows.Forms.Label();
			this.label15 = new System.Windows.Forms.Label();
			this.label14 = new System.Windows.Forms.Label();
			this.label27 = new System.Windows.Forms.Label();
			this.panelCencel = new System.Windows.Forms.Panel();
			this.label29 = new System.Windows.Forms.Label();
			this.label28 = new System.Windows.Forms.Label();
			this.panelOpen.SuspendLayout();
			this.panelSelect.SuspendLayout();
			this.groupBox2.SuspendLayout();
			this.panelPen.SuspendLayout();
			this.groupBox1.SuspendLayout();
			this.groupBox3.SuspendLayout();
			this.panelCencel.SuspendLayout();
			this.SuspendLayout();
			// 
			// pictureBox1
			// 
			this.pictureBox1.AllowDrop = true;
			this.pictureBox1.BackColor = System.Drawing.SystemColors.Window;
			this.pictureBox1.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.pictureBox1.Image = ((System.Drawing.Bitmap)(resources.GetObject("pictureBox1.Image")));
			this.pictureBox1.Location = new System.Drawing.Point(8, 224);
			this.pictureBox1.Name = "pictureBox1";
			this.pictureBox1.Size = new System.Drawing.Size(384, 384);
			this.pictureBox1.SizeMode = System.Windows.Forms.PictureBoxSizeMode.CenterImage;
			this.pictureBox1.TabIndex = 0;
			this.pictureBox1.TabStop = false;
			this.pictureBox1.DragEnter += new System.Windows.Forms.DragEventHandler(this.pictureBox1_DragEnter);
			this.pictureBox1.DragDrop += new System.Windows.Forms.DragEventHandler(this.pictureBox1_DragDrop);
			this.pictureBox1.MouseUp += new System.Windows.Forms.MouseEventHandler(this.pictureBox1_MouseUp);
			this.pictureBox1.MouseMove += new System.Windows.Forms.MouseEventHandler(this.pictureBox1_MouseMove);
			this.pictureBox1.MouseDown += new System.Windows.Forms.MouseEventHandler(this.pictureBox1_MouseDown);
			// 
			// buttonToolOpen
			// 
			this.buttonToolOpen.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.buttonToolOpen.Image = ((System.Drawing.Bitmap)(resources.GetObject("buttonToolOpen.Image")));
			this.buttonToolOpen.Location = new System.Drawing.Point(20, 4);
			this.buttonToolOpen.Name = "buttonToolOpen";
			this.buttonToolOpen.Size = new System.Drawing.Size(32, 28);
			this.buttonToolOpen.TabIndex = 1;
			this.buttonToolOpen.TabStop = false;
			this.toolTip1.SetToolTip(this.buttonToolOpen, "摜t@CJ(O)");
			this.buttonToolOpen.Click += new System.EventHandler(this.buttonToolOpen_Click);
			// 
			// radioButtonSelection
			// 
			this.radioButtonSelection.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioButtonSelection.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioButtonSelection.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioButtonSelection.Image")));
			this.radioButtonSelection.Location = new System.Drawing.Point(88, 4);
			this.radioButtonSelection.Name = "radioButtonSelection";
			this.radioButtonSelection.Size = new System.Drawing.Size(32, 28);
			this.radioButtonSelection.TabIndex = 9;
			this.toolTip1.SetToolTip(this.radioButtonSelection, "Ic[(S)");
			this.radioButtonSelection.CheckedChanged += new System.EventHandler(this.radioButtonSelection_CheckedChanged);
			// 
			// radioButtonPen
			// 
			this.radioButtonPen.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioButtonPen.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioButtonPen.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioButtonPen.Image")));
			this.radioButtonPen.Location = new System.Drawing.Point(68, 4);
			this.radioButtonPen.Name = "radioButtonPen";
			this.radioButtonPen.Size = new System.Drawing.Size(32, 28);
			this.radioButtonPen.TabIndex = 14;
			this.toolTip1.SetToolTip(this.radioButtonPen, "yc[(P)");
			// 
			// buttonCut
			// 
			this.buttonCut.BackColor = System.Drawing.SystemColors.Control;
			this.buttonCut.Location = new System.Drawing.Point(132, 64);
			this.buttonCut.Name = "buttonCut";
			this.buttonCut.Size = new System.Drawing.Size(56, 24);
			this.buttonCut.TabIndex = 19;
			this.buttonCut.TabStop = false;
			this.buttonCut.Text = "؂蔲";
			this.toolTip1.SetToolTip(this.buttonCut, "؂蔲(C)");
			this.buttonCut.Click += new System.EventHandler(this.buttonCut_Click);
			// 
			// buttonOK
			// 
			this.buttonOK.BackColor = System.Drawing.SystemColors.Control;
			this.buttonOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.buttonOK.Location = new System.Drawing.Point(136, 132);
			this.buttonOK.Name = "buttonOK";
			this.buttonOK.Size = new System.Drawing.Size(76, 24);
			this.buttonOK.TabIndex = 20;
			this.buttonOK.TabStop = false;
			this.buttonOK.Text = "ACR쐬";
			this.toolTip1.SetToolTip(this.buttonOK, "ACR쐬");
			this.buttonOK.Click += new System.EventHandler(this.buttonOK_Click);
			// 
			// toolTip1
			// 
			this.toolTip1.AutomaticDelay = 200;
			this.toolTip1.ShowAlways = true;
			// 
			// buttonCancel
			// 
			this.buttonCancel.BackColor = System.Drawing.SystemColors.Control;
			this.buttonCancel.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.buttonCancel.Location = new System.Drawing.Point(160, 4);
			this.buttonCancel.Name = "buttonCancel";
			this.buttonCancel.Size = new System.Drawing.Size(64, 24);
			this.buttonCancel.TabIndex = 9;
			this.buttonCancel.TabStop = false;
			this.buttonCancel.Text = "LZ";
			this.toolTip1.SetToolTip(this.buttonCancel, "LZ");
			this.buttonCancel.Click += new System.EventHandler(this.buttonCancel_Click);
			// 
			// buttonOpen2
			// 
			this.buttonOpen2.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.buttonOpen2.Image = ((System.Drawing.Bitmap)(resources.GetObject("buttonOpen2.Image")));
			this.buttonOpen2.Location = new System.Drawing.Point(140, 8);
			this.buttonOpen2.Name = "buttonOpen2";
			this.buttonOpen2.Size = new System.Drawing.Size(32, 28);
			this.buttonOpen2.TabIndex = 25;
			this.buttonOpen2.TabStop = false;
			this.toolTip1.SetToolTip(this.buttonOpen2, "摜t@CJ(O)");
			this.buttonOpen2.Click += new System.EventHandler(this.buttonToolOpen_Click);
			// 
			// buttonBack
			// 
			this.buttonBack.BackColor = System.Drawing.SystemColors.Control;
			this.buttonBack.Location = new System.Drawing.Point(136, 8);
			this.buttonBack.Name = "buttonBack";
			this.buttonBack.Size = new System.Drawing.Size(96, 24);
			this.buttonBack.TabIndex = 39;
			this.buttonBack.TabStop = false;
			this.buttonBack.Text = "I͈͂ύX";
			this.toolTip1.SetToolTip(this.buttonBack, "I͈͂ύX");
			this.buttonBack.Click += new System.EventHandler(this.buttonBack_Click);
			// 
			// buttonToolUndo
			// 
			this.buttonToolUndo.BackColor = System.Drawing.SystemColors.Window;
			this.buttonToolUndo.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.buttonToolUndo.Image = ((System.Drawing.Bitmap)(resources.GetObject("buttonToolUndo.Image")));
			this.buttonToolUndo.Location = new System.Drawing.Point(148, 100);
			this.buttonToolUndo.Name = "buttonToolUndo";
			this.buttonToolUndo.Size = new System.Drawing.Size(32, 28);
			this.buttonToolUndo.TabIndex = 49;
			this.buttonToolUndo.TabStop = false;
			this.toolTip1.SetToolTip(this.buttonToolUndo, "(U)");
			this.buttonToolUndo.Click += new System.EventHandler(this.buttonToolUndo_Click);
			// 
			// buttonToolRedo
			// 
			this.buttonToolRedo.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.buttonToolRedo.Image = ((System.Drawing.Bitmap)(resources.GetObject("buttonToolRedo.Image")));
			this.buttonToolRedo.Location = new System.Drawing.Point(268, 100);
			this.buttonToolRedo.Name = "buttonToolRedo";
			this.buttonToolRedo.Size = new System.Drawing.Size(32, 28);
			this.buttonToolRedo.TabIndex = 50;
			this.buttonToolRedo.TabStop = false;
			this.toolTip1.SetToolTip(this.buttonToolRedo, "蒼(R)");
			this.buttonToolRedo.Click += new System.EventHandler(this.buttonToolRedo_Click);
			// 
			// radioGridNone
			// 
			this.radioGridNone.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioGridNone.BackColor = System.Drawing.SystemColors.Window;
			this.radioGridNone.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioGridNone.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioGridNone.Image")));
			this.radioGridNone.Location = new System.Drawing.Point(112, 68);
			this.radioGridNone.Name = "radioGridNone";
			this.radioGridNone.Size = new System.Drawing.Size(32, 28);
			this.radioGridNone.TabIndex = 38;
			this.toolTip1.SetToolTip(this.radioGridNone, "Obh\(G)");
			this.radioGridNone.CheckedChanged += new System.EventHandler(this.radioGridNone_CheckedChanged);
			// 
			// radioGridBlack
			// 
			this.radioGridBlack.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioGridBlack.BackColor = System.Drawing.SystemColors.Window;
			this.radioGridBlack.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioGridBlack.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioGridBlack.Image")));
			this.radioGridBlack.Location = new System.Drawing.Point(184, 68);
			this.radioGridBlack.Name = "radioGridBlack";
			this.radioGridBlack.Size = new System.Drawing.Size(32, 28);
			this.radioGridBlack.TabIndex = 39;
			this.toolTip1.SetToolTip(this.radioGridBlack, "Obhŕ\(B)");
			this.radioGridBlack.CheckedChanged += new System.EventHandler(this.radioGridBlack_CheckedChanged);
			// 
			// radioGridWhite
			// 
			this.radioGridWhite.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioGridWhite.BackColor = System.Drawing.SystemColors.Window;
			this.radioGridWhite.Checked = true;
			this.radioGridWhite.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioGridWhite.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioGridWhite.Image")));
			this.radioGridWhite.Location = new System.Drawing.Point(256, 68);
			this.radioGridWhite.Name = "radioGridWhite";
			this.radioGridWhite.Size = new System.Drawing.Size(32, 28);
			this.radioGridWhite.TabIndex = 40;
			this.radioGridWhite.TabStop = true;
			this.toolTip1.SetToolTip(this.radioGridWhite, "Obh𔒂ŕ\(W)");
			this.radioGridWhite.CheckedChanged += new System.EventHandler(this.radioGridWhite_CheckedChanged);
			// 
			// radioPen3
			// 
			this.radioPen3.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioPen3.BackColor = System.Drawing.SystemColors.Control;
			this.radioPen3.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioPen3.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioPen3.Image")));
			this.radioPen3.Location = new System.Drawing.Point(60, 8);
			this.radioPen3.Name = "radioPen3";
			this.radioPen3.Size = new System.Drawing.Size(24, 24);
			this.radioPen3.TabIndex = 20;
			this.toolTip1.SetToolTip(this.radioPen3, "3x3sNZ (3)");
			this.radioPen3.CheckedChanged += new System.EventHandler(this.radioPen3_CheckedChanged);
			// 
			// radioPen2
			// 
			this.radioPen2.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioPen2.BackColor = System.Drawing.SystemColors.Control;
			this.radioPen2.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioPen2.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioPen2.Image")));
			this.radioPen2.Location = new System.Drawing.Point(32, 8);
			this.radioPen2.Name = "radioPen2";
			this.radioPen2.Size = new System.Drawing.Size(24, 24);
			this.radioPen2.TabIndex = 19;
			this.toolTip1.SetToolTip(this.radioPen2, "2x2sNZ (2)");
			this.radioPen2.CheckedChanged += new System.EventHandler(this.radioPen2_CheckedChanged);
			// 
			// radioPen1
			// 
			this.radioPen1.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioPen1.BackColor = System.Drawing.SystemColors.Control;
			this.radioPen1.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioPen1.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioPen1.Image")));
			this.radioPen1.Location = new System.Drawing.Point(4, 8);
			this.radioPen1.Name = "radioPen1";
			this.radioPen1.Size = new System.Drawing.Size(24, 24);
			this.radioPen1.TabIndex = 18;
			this.toolTip1.SetToolTip(this.radioPen1, "1x1sNZ (1)");
			this.radioPen1.CheckedChanged += new System.EventHandler(this.radioPen1_CheckedChanged);
			// 
			// radioPen4
			// 
			this.radioPen4.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioPen4.BackColor = System.Drawing.SystemColors.Control;
			this.radioPen4.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioPen4.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioPen4.Image")));
			this.radioPen4.Location = new System.Drawing.Point(88, 8);
			this.radioPen4.Name = "radioPen4";
			this.radioPen4.Size = new System.Drawing.Size(24, 24);
			this.radioPen4.TabIndex = 21;
			this.toolTip1.SetToolTip(this.radioPen4, "4x4sNZ (4)");
			this.radioPen4.CheckedChanged += new System.EventHandler(this.radioPen4_CheckedChanged);
			// 
			// radioAreaBlack
			// 
			this.radioAreaBlack.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioAreaBlack.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioAreaBlack.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioAreaBlack.Image")));
			this.radioAreaBlack.Location = new System.Drawing.Point(152, 32);
			this.radioAreaBlack.Name = "radioAreaBlack";
			this.radioAreaBlack.Size = new System.Drawing.Size(32, 28);
			this.radioAreaBlack.TabIndex = 32;
			this.toolTip1.SetToolTip(this.radioAreaBlack, "Ïŕ\(B)");
			this.radioAreaBlack.CheckedChanged += new System.EventHandler(this.radioAreaBlack_CheckedChanged);
			// 
			// radioAreaWhite
			// 
			this.radioAreaWhite.Appearance = System.Windows.Forms.Appearance.Button;
			this.radioAreaWhite.FlatStyle = System.Windows.Forms.FlatStyle.Flat;
			this.radioAreaWhite.Image = ((System.Drawing.Bitmap)(resources.GetObject("radioAreaWhite.Image")));
			this.radioAreaWhite.Location = new System.Drawing.Point(220, 32);
			this.radioAreaWhite.Name = "radioAreaWhite";
			this.radioAreaWhite.Size = new System.Drawing.Size(32, 28);
			this.radioAreaWhite.TabIndex = 31;
			this.toolTip1.SetToolTip(this.radioAreaWhite, "Ï𔒂ŕ\(W)");
			this.radioAreaWhite.CheckedChanged += new System.EventHandler(this.radioAreaWhite_CheckedChanged);
			// 
			// panelOpen
			// 
			this.panelOpen.BackColor = System.Drawing.SystemColors.Window;
			this.panelOpen.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.label4,
																					this.label3,
																					this.label2,
																					this.label1,
																					this.buttonToolOpen});
			this.panelOpen.Location = new System.Drawing.Point(8, 0);
			this.panelOpen.Name = "panelOpen";
			this.panelOpen.Size = new System.Drawing.Size(384, 188);
			this.panelOpen.TabIndex = 23;
			// 
			// label4
			// 
			this.label4.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label4.Location = new System.Drawing.Point(172, 40);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(84, 24);
			this.label4.TabIndex = 3;
			this.label4.Text = "ĂD";
			// 
			// label3
			// 
			this.label3.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label3.ForeColor = System.Drawing.Color.Black;
			this.label3.Location = new System.Drawing.Point(72, 40);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(96, 24);
			this.label3.TabIndex = 2;
			this.label3.Text = "hbOhbv";
			// 
			// label2
			// 
			this.label2.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label2.Location = new System.Drawing.Point(20, 40);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(52, 24);
			this.label2.TabIndex = 1;
			this.label2.Text = "t@C";
			// 
			// label1
			// 
			this.label1.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label1.Location = new System.Drawing.Point(60, 12);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(196, 20);
			this.label1.TabIndex = 0;
			this.label1.Text = "ĉ摜t@CI邩";
			// 
			// panelSelect
			// 
			this.panelSelect.BackColor = System.Drawing.SystemColors.Window;
			this.panelSelect.Controls.AddRange(new System.Windows.Forms.Control[] {
																					  this.radioAreaBlack,
																					  this.radioAreaWhite,
																					  this.label11,
																					  this.label13,
																					  this.label12,
																					  this.groupBox2,
																					  this.label10,
																					  this.label9,
																					  this.label6,
																					  this.radioButtonSelection,
																					  this.label5,
																					  this.buttonCut});
			this.panelSelect.Location = new System.Drawing.Point(8, 0);
			this.panelSelect.Name = "panelSelect";
			this.panelSelect.Size = new System.Drawing.Size(384, 192);
			this.panelSelect.TabIndex = 24;
			this.panelSelect.Visible = false;
			// 
			// label11
			// 
			this.label11.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label11.Location = new System.Drawing.Point(36, 40);
			this.label11.Name = "label11";
			this.label11.Size = new System.Drawing.Size(116, 20);
			this.label11.TabIndex = 33;
			this.label11.Text = "I͈͂̐F́C";
			// 
			// label13
			// 
			this.label13.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label13.Location = new System.Drawing.Point(260, 40);
			this.label13.Name = "label13";
			this.label13.Size = new System.Drawing.Size(104, 20);
			this.label13.TabIndex = 35;
			this.label13.Text = "ɕύXł܂D";
			// 
			// label12
			// 
			this.label12.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label12.Location = new System.Drawing.Point(188, 40);
			this.label12.Name = "label12";
			this.label12.Size = new System.Drawing.Size(32, 20);
			this.label12.TabIndex = 34;
			this.label12.Text = "Ɣ";
			// 
			// groupBox2
			// 
			this.groupBox2.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.buttonOpen2,
																					this.label7,
																					this.label32,
																					this.label8,
																					this.label30,
																					this.label31});
			this.groupBox2.Location = new System.Drawing.Point(4, 124);
			this.groupBox2.Name = "groupBox2";
			this.groupBox2.Size = new System.Drawing.Size(376, 68);
			this.groupBox2.TabIndex = 30;
			this.groupBox2.TabStop = false;
			// 
			// label7
			// 
			this.label7.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label7.Location = new System.Drawing.Point(160, 48);
			this.label7.Name = "label7";
			this.label7.Size = new System.Drawing.Size(84, 16);
			this.label7.TabIndex = 28;
			this.label7.Text = "ĂD";
			// 
			// label32
			// 
			this.label32.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label32.Location = new System.Drawing.Point(8, 16);
			this.label32.Name = "label32";
			this.label32.Size = new System.Drawing.Size(140, 24);
			this.label32.TabIndex = 29;
			this.label32.Text = "摜Iђꍇ";
			// 
			// label8
			// 
			this.label8.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label8.ForeColor = System.Drawing.Color.Black;
			this.label8.Location = new System.Drawing.Point(60, 48);
			this.label8.Name = "label8";
			this.label8.Size = new System.Drawing.Size(96, 16);
			this.label8.TabIndex = 27;
			this.label8.Text = "hbOhbv";
			// 
			// label30
			// 
			this.label30.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label30.Location = new System.Drawing.Point(8, 48);
			this.label30.Name = "label30";
			this.label30.Size = new System.Drawing.Size(52, 16);
			this.label30.TabIndex = 26;
			this.label30.Text = "t@C";
			// 
			// label31
			// 
			this.label31.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label31.Location = new System.Drawing.Point(180, 16);
			this.label31.Name = "label31";
			this.label31.Size = new System.Drawing.Size(188, 20);
			this.label31.TabIndex = 24;
			this.label31.Text = "NbNăt@CI邩C";
			// 
			// label10
			// 
			this.label10.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label10.Location = new System.Drawing.Point(196, 68);
			this.label10.Name = "label10";
			this.label10.Size = new System.Drawing.Size(100, 20);
			this.label10.TabIndex = 20;
			this.label10.Text = "ĂD";
			// 
			// label9
			// 
			this.label9.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label9.Location = new System.Drawing.Point(20, 68);
			this.label9.Name = "label9";
			this.label9.Size = new System.Drawing.Size(108, 20);
			this.label9.TabIndex = 13;
			this.label9.Text = "IC";
			// 
			// label6
			// 
			this.label6.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label6.Location = new System.Drawing.Point(124, 12);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(188, 20);
			this.label6.TabIndex = 10;
			this.label6.Text = "pāCIĂD";
			// 
			// label5
			// 
			this.label5.Font = new System.Drawing.Font("MS UI Gothic", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label5.Location = new System.Drawing.Point(20, 12);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(64, 20);
			this.label5.TabIndex = 0;
			this.label5.Text = "Ic[";
			// 
			// panelPen
			// 
			this.panelPen.BackColor = System.Drawing.SystemColors.Window;
			this.panelPen.Controls.AddRange(new System.Windows.Forms.Control[] {
																				   this.buttonToolUndo,
																				   this.label25,
																				   this.label24,
																				   this.label23,
																				   this.buttonToolRedo,
																				   this.label20,
																				   this.label17,
																				   this.label21,
																				   this.radioGridNone,
																				   this.label22,
																				   this.radioGridBlack,
																				   this.label19,
																				   this.label18,
																				   this.label16,
																				   this.radioGridWhite,
																				   this.groupBox1,
																				   this.groupBox3,
																				   this.label26,
																				   this.label15,
																				   this.label14,
																				   this.radioButtonPen,
																				   this.buttonOK,
																				   this.label27});
			this.panelPen.Location = new System.Drawing.Point(8, 0);
			this.panelPen.Name = "panelPen";
			this.panelPen.Size = new System.Drawing.Size(384, 192);
			this.panelPen.TabIndex = 25;
			this.panelPen.Visible = false;
			// 
			// label25
			// 
			this.label25.BackColor = System.Drawing.SystemColors.Window;
			this.label25.Location = new System.Drawing.Point(304, 108);
			this.label25.Name = "label25";
			this.label25.Size = new System.Drawing.Size(76, 23);
			this.label25.TabIndex = 53;
			this.label25.Text = "𗘗p܂D";
			// 
			// label24
			// 
			this.label24.BackColor = System.Drawing.SystemColors.Window;
			this.label24.Location = new System.Drawing.Point(184, 108);
			this.label24.Name = "label24";
			this.label24.Size = new System.Drawing.Size(84, 23);
			this.label24.TabIndex = 52;
			this.label24.Text = "C蒼ɂ";
			// 
			// label23
			// 
			this.label23.BackColor = System.Drawing.SystemColors.Window;
			this.label23.Location = new System.Drawing.Point(20, 108);
			this.label23.Name = "label23";
			this.label23.Size = new System.Drawing.Size(124, 23);
			this.label23.TabIndex = 51;
			this.label23.Text = "`ɂ";
			// 
			// label20
			// 
			this.label20.BackColor = System.Drawing.SystemColors.Window;
			this.label20.Location = new System.Drawing.Point(148, 76);
			this.label20.Name = "label20";
			this.label20.Size = new System.Drawing.Size(32, 23);
			this.label20.TabIndex = 46;
			this.label20.Text = "F";
			// 
			// label17
			// 
			this.label17.BackColor = System.Drawing.SystemColors.Window;
			this.label17.ForeColor = System.Drawing.Color.FromArgb(((System.Byte)(0)), ((System.Byte)(0)), ((System.Byte)(192)));
			this.label17.Location = new System.Drawing.Point(92, 44);
			this.label17.Name = "label17";
			this.label17.Size = new System.Drawing.Size(40, 23);
			this.label17.TabIndex = 42;
			this.label17.Text = "4iK";
			// 
			// label21
			// 
			this.label21.BackColor = System.Drawing.SystemColors.Window;
			this.label21.Location = new System.Drawing.Point(220, 76);
			this.label21.Name = "label21";
			this.label21.Size = new System.Drawing.Size(30, 23);
			this.label21.TabIndex = 47;
			this.label21.Text = "F";
			// 
			// label22
			// 
			this.label22.BackColor = System.Drawing.SystemColors.Window;
			this.label22.Location = new System.Drawing.Point(292, 76);
			this.label22.Name = "label22";
			this.label22.Size = new System.Drawing.Size(88, 23);
			this.label22.TabIndex = 48;
			this.label22.Text = "ɕύXł܂D";
			// 
			// label19
			// 
			this.label19.BackColor = System.Drawing.SystemColors.Window;
			this.label19.Location = new System.Drawing.Point(20, 76);
			this.label19.Name = "label19";
			this.label19.Size = new System.Drawing.Size(88, 23);
			this.label19.TabIndex = 45;
			this.label19.Text = "Obh͔\";
			// 
			// label18
			// 
			this.label18.BackColor = System.Drawing.SystemColors.Window;
			this.label18.Location = new System.Drawing.Point(252, 44);
			this.label18.Name = "label18";
			this.label18.Size = new System.Drawing.Size(88, 23);
			this.label18.TabIndex = 44;
			this.label18.Text = "ɕύXł܂D";
			// 
			// label16
			// 
			this.label16.BackColor = System.Drawing.SystemColors.Window;
			this.label16.Location = new System.Drawing.Point(20, 44);
			this.label16.Name = "label16";
			this.label16.Size = new System.Drawing.Size(72, 23);
			this.label16.TabIndex = 41;
			this.label16.Text = "y̑";
			// 
			// groupBox1
			// 
			this.groupBox1.BackColor = System.Drawing.SystemColors.Window;
			this.groupBox1.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.radioPen3,
																					this.radioPen2,
																					this.radioPen1,
																					this.radioPen4});
			this.groupBox1.Location = new System.Drawing.Point(132, 32);
			this.groupBox1.Name = "groupBox1";
			this.groupBox1.Size = new System.Drawing.Size(116, 36);
			this.groupBox1.TabIndex = 43;
			this.groupBox1.TabStop = false;
			// 
			// groupBox3
			// 
			this.groupBox3.Controls.AddRange(new System.Windows.Forms.Control[] {
																					this.label34,
																					this.buttonBack,
																					this.label33});
			this.groupBox3.Location = new System.Drawing.Point(4, 156);
			this.groupBox3.Name = "groupBox3";
			this.groupBox3.Size = new System.Drawing.Size(380, 36);
			this.groupBox3.TabIndex = 33;
			this.groupBox3.TabStop = false;
			// 
			// label34
			// 
			this.label34.Location = new System.Drawing.Point(244, 16);
			this.label34.Name = "label34";
			this.label34.Size = new System.Drawing.Size(94, 12);
			this.label34.TabIndex = 40;
			this.label34.Text = "ĂD";
			// 
			// label33
			// 
			this.label33.Location = new System.Drawing.Point(4, 16);
			this.label33.Name = "label33";
			this.label33.Size = new System.Drawing.Size(132, 12);
			this.label33.TabIndex = 38;
			this.label33.Text = "I͈͂Iђɂ";
			// 
			// label26
			// 
			this.label26.Font = new System.Drawing.Font("MS UI Gothic", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label26.Location = new System.Drawing.Point(8, 140);
			this.label26.Name = "label26";
			this.label26.Size = new System.Drawing.Size(124, 16);
			this.label26.TabIndex = 28;
			this.label26.Text = "hԂC";
			// 
			// label15
			// 
			this.label15.Location = new System.Drawing.Point(104, 12);
			this.label15.Name = "label15";
			this.label15.Size = new System.Drawing.Size(192, 23);
			this.label15.TabIndex = 15;
			this.label15.Text = "pāCwihԂĂD";
			// 
			// label14
			// 
			this.label14.Location = new System.Drawing.Point(8, 12);
			this.label14.Name = "label14";
			this.label14.Size = new System.Drawing.Size(60, 23);
			this.label14.TabIndex = 0;
			this.label14.Text = "yc[";
			// 
			// label27
			// 
			this.label27.Location = new System.Drawing.Point(220, 140);
			this.label27.Name = "label27";
			this.label27.Size = new System.Drawing.Size(124, 16);
			this.label27.TabIndex = 29;
			this.label27.Text = "ĂD";
			// 
			// panelCencel
			// 
			this.panelCencel.BackColor = System.Drawing.SystemColors.Window;
			this.panelCencel.Controls.AddRange(new System.Windows.Forms.Control[] {
																					  this.label29,
																					  this.label28,
																					  this.buttonCancel});
			this.panelCencel.Location = new System.Drawing.Point(8, 192);
			this.panelCencel.Name = "panelCencel";
			this.panelCencel.Size = new System.Drawing.Size(384, 28);
			this.panelCencel.TabIndex = 31;
			// 
			// label29
			// 
			this.label29.BackColor = System.Drawing.SystemColors.Window;
			this.label29.Font = new System.Drawing.Font("MS UI Gothic", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label29.Location = new System.Drawing.Point(232, 8);
			this.label29.Name = "label29";
			this.label29.Size = new System.Drawing.Size(92, 16);
			this.label29.TabIndex = 31;
			this.label29.Text = "ĂD";
			// 
			// label28
			// 
			this.label28.BackColor = System.Drawing.SystemColors.Window;
			this.label28.Font = new System.Drawing.Font("MS UI Gothic", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(128)));
			this.label28.Location = new System.Drawing.Point(8, 8);
			this.label28.Name = "label28";
			this.label28.Size = new System.Drawing.Size(148, 16);
			this.label28.TabIndex = 30;
			this.label28.Text = "ACR쐬߂ꍇ́C";
			// 
			// IconForm
			// 
			this.AllowDrop = true;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 12);
			this.ClientSize = new System.Drawing.Size(402, 608);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.panelCencel,
																		  this.pictureBox1,
																		  this.panelPen,
																		  this.panelOpen,
																		  this.panelSelect});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.KeyPreview = true;
			this.Name = "IconForm";
			this.Text = "ACR쐬܂";
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.IconForm_KeyDown);
			this.DragDrop += new System.Windows.Forms.DragEventHandler(this.iconForm_DragDrop);
			this.DragEnter += new System.Windows.Forms.DragEventHandler(this.iconForm_DragEnter);
			this.panelOpen.ResumeLayout(false);
			this.panelSelect.ResumeLayout(false);
			this.groupBox2.ResumeLayout(false);
			this.panelPen.ResumeLayout(false);
			this.groupBox1.ResumeLayout(false);
			this.groupBox3.ResumeLayout(false);
			this.panelCencel.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region Methods for Checking Button Envents 
		private void buttonToolOpen_Click(object sender, System.EventArgs e)
		{
			OpenImageDialog();
		}

		private void buttonOK_Click(object sender, System.EventArgs e)
		{
			if(clipBmp != null)
			{
				CreateFaceIcon();
				this.Close();
			}
			else
			{
				MessageBox.Show("ACR̍쐬͊Ă܂D");
				return;
			}	
		}

		private void buttonCancel_Click(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void radioButtonSelection_CheckedChanged(object sender, System.EventArgs e)
		{
			mode.tool = ToolMode.SELECT;
			pictureBox1.Cursor = Cursors.Cross;
		}

		private void radioButtonMove_CheckedChanged(object sender, System.EventArgs e)
		{
			mode.tool = ToolMode.MOVE;
			pictureBox1.Cursor = Cursors.SizeAll;
		}

		private void radioButtonPen_CheckedChanged(object sender, System.EventArgs e)
		{
			mode.tool = ToolMode.PEN;
			pictureBox1.Cursor = Cursors.Arrow;
		}

		private void buttonToolUndo_Click(object sender, System.EventArgs e)
		{	
			UndoStroke();	
		}

		private void buttonToolRedo_Click(object sender, System.EventArgs e)
		{
			RedoStroke();
		}
		private void buttonCut_Click(object sender, System.EventArgs e)
		{
			ClipSelection();
			return;
		}

		private void radioAreaBlack_CheckedChanged(object sender, System.EventArgs e)
		{
			mode.tool = ToolMode.SELECT;
			selectRegion.Pen.Color = Color.Black;
			selectRegion.PenBold.Color = Color.Black;
			UpdateGraphics();
			return;
		}

		private void radioAreaWhite_CheckedChanged(object sender, System.EventArgs e)
		{
			mode.tool = ToolMode.SELECT;
			selectRegion.Pen.Color = Color.White;
			selectRegion.PenBold.Color = Color.White;
			UpdateGraphics();
			return;		
		}

		private void radioGridNone_CheckedChanged(object sender, System.EventArgs e)
		{
			mode.tool = ToolMode.PEN;
			grid.visible = false;
			UpdateGraphics();
			return;
		}

		private void radioGridBlack_CheckedChanged(object sender, System.EventArgs e)
		{
			mode.tool = ToolMode.PEN;
			grid.visible = true;

			grid.pen = new Pen(Color.Black, 0.5F);
			UpdateGraphics();
			return;

		}

		private void radioGridWhite_CheckedChanged(object sender, System.EventArgs e)
		{
				
			mode.tool = ToolMode.PEN;
			grid.visible = true;
			grid.pen = new Pen(Color.White, 0.5F);
			UpdateGraphics();
			return;
		
		}

		private void radioPen1_CheckedChanged(object sender, System.EventArgs e)
		{
			penSize = 1;
			UpdateGraphics();
		}

		private void radioPen2_CheckedChanged(object sender, System.EventArgs e)
		{
			penSize = 2;
			UpdateGraphics();
		}

		private void radioPen3_CheckedChanged(object sender, System.EventArgs e)
		{
			penSize = 3;
			UpdateGraphics();
		}

		private void radioPen4_CheckedChanged(object sender, System.EventArgs e)
		{
			penSize = 4;
			UpdateGraphics();
		}

		private void buttonBack_Click(object sender, System.EventArgs e)
		{
			mode.tool = ToolMode.SELECT;
			ChangeFormState(mode.tool);
			pictureBox1.Image = new Bitmap(resizeBmp);
			graphics = Graphics.FromImage(pictureBox1.Image);
			graphics.InterpolationMode = InterpolationMode.HighQualityBicubic;	//⊮[h̐ݒ
			UpdateGraphics();
		}
		#endregion

		#region Methods for Drag and Drop

		private void iconForm_DragDrop(object sender, System.Windows.Forms.DragEventArgs e)
		{
			string[] filenames = (string[])e.Data.GetData(DataFormats.FileDrop);
			if (filenames.Length > 1) 
				MessageBox.Show("t@C͂ЂƂhbOAhhbvĂ");
			else 
			{
				LoadImage(filenames[0]);
				mode.tool = ToolMode.SELECT;
				ChangeFormState(mode.tool);
			}
		}

		private void iconForm_DragEnter(object sender, System.Windows.Forms.DragEventArgs e)
		{
			if (e.Data.GetDataPresent(DataFormats.FileDrop))
			{ 
				e.Effect = DragDropEffects.Copy;
			}
			else
			{
				e.Effect = DragDropEffects.None;
			}
		}


		private void pictureBox1_DragDrop(object sender, System.Windows.Forms.DragEventArgs e)
		{
			string[] filenames = (string[])e.Data.GetData(DataFormats.FileDrop);
			if (filenames.Length > 1) 
				MessageBox.Show("t@C͂ЂƂhbOAhhbvĂ");
			else 
			{
				LoadImage(filenames[0]);
				mode.tool = ToolMode.SELECT;
				ChangeFormState(mode.tool);
			}	
		}

		private void pictureBox1_DragEnter(object sender, System.Windows.Forms.DragEventArgs e)
		{
			if (e.Data.GetDataPresent(DataFormats.FileDrop))
			{ 
				e.Effect = DragDropEffects.Copy;
			}
			else
			{
				e.Effect = DragDropEffects.None;
			}
		}

		#endregion

	}
}
